/*
 * IRC - Internet Relay Chat, ircd/m_hidehost.c
 * Written by David Herrmann.
 */

/*
 * m_functions execute protocol messages on this server:
 *
 *    cptr    is always NON-NULL, pointing to a *LOCAL* client
 *            structure (with an open socket connected!). This
 *            identifies the physical socket where the message
 *            originated (or which caused the m_function to be
 *            executed--some m_functions may call others...).
 *
 *    sptr    is the source of the message, defined by the
 *            prefix part of the message if present. If not
 *            or prefix not found, then sptr==cptr.
 *
 *            (!IsServer(cptr)) => (cptr == sptr), because
 *            prefixes are taken *only* from servers...
 *
 *            (IsServer(cptr))
 *                    (sptr == cptr) => the message didn't
 *                    have the prefix.
 *
 *                    (sptr != cptr && IsServer(sptr) means
 *                    the prefix specified servername. (?)
 *
 *                    (sptr != cptr && !IsServer(sptr) means
 *                    that message originated from a remote
 *                    user (not local).
 *
 *            combining
 *
 *            (!IsServer(sptr)) means that, sptr can safely
 *            taken as defining the target structure of the
 *            message in this server.
 *
 *    *Always* true (if 'parse' and others are working correct):
 *
 *    1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *    2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *            *cannot* be a local connection, unless it's
 *            actually cptr!). [MyConnect(x) should probably
 *            be defined as (x == x->from) --msa ]
 *
 *    parc    number of variable parameter strings (if zero,
 *            parv is allowed to be NULL)
 *
 *    parv    a NULL terminated list of parameter pointers,
 *
 *                    parv[0], sender (prefix string), if not present
 *                            this points to an empty string.
 *                    parv[1]...parv[parc-1]
 *                            pointers to additional parameters
 *                    parv[parc] == NULL, *always*
 *
 *            note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                    non-NULL pointers.
 */
#include "config.h"

#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_log.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "match.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "s_conf.h"
#include "s_misc.h"
#include "s_user.h"
#include "send.h"
#include "sys.h"

/* ms_hidehost
 *  parv[0] = sender
 *  parv[1] = target nick
 */
/** Remote hidehost
 * Allows ulined servers to force umode +x on remote users. Though, we should use numerics
 * as target, the current services use nicknames. This may be changed in future.
 *
 * The HIDEHOST request can be generated by EVERY server. It is forwarded to the server
 * of the user which then sets the umode +x and broadcasts the new umodes.
 */
int ms_hidehost(struct Client *cptr, struct Client *sptr, int parc, char *parv[]) {
    signed int propagate;
    struct Client *acptr;
    struct Flags setflags;

    if(parc < 2) {
        return need_more_params(sptr, "HIDEHOST");
    }

    /* Check whether the user is already gone or already hidden. */
    if(!(acptr = FindUser(parv[1])) || !IsUser(acptr) || IsHiddenHost(acptr)) {
        return 0;
    }

    /* If the user is no local user, we forward the message to the user's server. */
    if(!MyConnect(acptr)) {
        sendcmdto_one(sptr, CMD_HIDEHOST, cli_user(acptr)->server, "%s", cli_name(acptr));
        return 0;
    }

    /* Set +x and propagate the changed modes. */
    propagate = !!HasPriv(acptr, PRIV_PROPAGATE);
    setflags = cli_flags(acptr);
    hide_hostmask(acptr, FLAG_HIDDENHOST);
    send_umode_out(acptr, acptr, &setflags, propagate);

    return 0;
}

